import { DI } from '@microsoft/fast-foundation';
import { logger } from './logger';
/**
 * URLTargeting client filter function.
 * @remarks
 * This will return true when the current URL matches on ANY of the parameters. If you wish to match on multiple
 * parameters, consider chaining an array of urlTargeting filters together.
 * @param url - A url.
 * @param pattern - A regex.
 * @param schemes - Schemes/protocols to target.
 * @param hosts - Hosts to target.
 * @param paths - Paths to target.
 * @public
 * @privateRemarks
 * The pattern regex can cover everything here, but some devs will likely welcome the explicit arguments.
 */
export function urlTargetingFilter(url, pattern, schemes, hosts, paths) {
    if (!url) {
        logger.warn('URLTargeting filter needs a url to filter on.');
        return true;
    }
    if (!pattern && !schemes && !hosts && !paths) {
        logger.warn('URLTargeting filter parameters must include pattern, schemes, hosts, or paths.');
        return true;
    }
    if (pattern && pattern.test(url)) {
        return true;
    }
    let urlObj;
    try {
        urlObj = new URL(url);
    }
    catch (e) {
        return true;
    }
    if (schemes && schemes.includes(urlObj.protocol?.slice(0, -1))) {
        return true;
    }
    if (hosts && hosts.includes(urlObj.host)) {
        return true;
    }
    return !!paths && paths.includes(urlObj.pathname);
}
/**
 * Default URLTargeting DI implementation.
 * @public
 */
export class DefaultURLTargeting {
    /** {@inheritDoc ClientFilter.filter} */
    filter(params) {
        return urlTargetingFilter(params?.url, params?.pattern, params?.schemes, params?.hosts, params?.paths);
    }
    /** {@inheritDoc URLTargeting.isSecure} */
    isSecure(url) {
        return urlTargetingFilter(url, null, ['https', 'wss']);
    }
}
/**
 * URLTargeting DI key.
 *
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 *
 * @internal
 */
export const URLTargeting = DI.createInterface((x) => x.transient(DefaultURLTargeting));
